/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol". 
 */

package com.funambol.android;

import android.accounts.Account;
import android.accounts.AccountManager;
import android.content.BroadcastReceiver;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;

import com.funambol.android.activities.AndroidDisplayManager;
import com.funambol.android.controller.AndroidController;
import com.funambol.client.controller.AboutScreenController;
import com.funambol.client.controller.HomeScreenController;
import com.funambol.client.controller.SyncSettingsScreenController;
import com.funambol.client.ui.Screen;

/**
 * A BroadcastReceiver implementation sensible to account changes
 */
public class AndroidAccountManager extends BroadcastReceiver {

    private final String TAG_LOG = "AndroidAccountReceiver";
    
    private AndroidDisplayManager dm = new AndroidDisplayManager();

    private static boolean ignoreAccountChanges = false;

    /**
     * Implementation of the super class abstract method: define what tpo do
     * when this BroadcastReceiver is invoked.
     * @param context the Context object
     * @param intent the Intent action to be triggered
     */
    public void onReceive(Context context, Intent intent) {

        //Does anything if another operation is pending
        if(ignoreAccountChanges) {
            return;
        }

        //Get the native account reference
        Account account = AndroidController.getNativeAccount();

        if(account == null && AndroidController.isInitialized()) {
            AndroidController controller = AndroidController.getInstance();

            AboutScreenController aboutController = controller.getAboutScreenController();
            if(aboutController != null) {
                hideScreen(aboutController.getAboutScreen());
            }

            SyncSettingsScreenController settingsController = controller.getSyncSettingsScreenController();
            if(settingsController != null) {
                hideScreen(settingsController.getSyncSettingsScreen());
            }

            HomeScreenController homeController = controller.getHomeScreenController();
            if(homeController != null) {
                hideScreen(homeController.getHomeScreen());
            }

            SharedPreferences settings = context.getSharedPreferences(
                    AndroidConfiguration.KEY_FUNAMBOL_PREFERENCES, 0);
            SharedPreferences.Editor editor = settings.edit();
            editor.clear();
            editor.commit();

            // Dispose all the singletons
            AppInitializer             .dispose();
            AndroidController          .dispose();
            AndroidConfiguration       .dispose();
            AndroidCustomization       .dispose();
            AndroidLocalization        .dispose();
            AndroidAppSyncSourceManager.dispose();
        }
    }

    /**
     * Use the DisplayManager to hide the given screen
     * @param s is the Screen to be hidden
     */
    private void hideScreen(Screen s) {
        try {
            if(s != null) dm.hideScreen(s);
        } catch(Exception ex) {
            android.util.Log.e(TAG_LOG, "Error while closing screen: " + ex);
        }
    }

    /**
     * When an operation has begun on an account advise the receiver to ignore
     * any other concurrent change
     */
    public static void beginAccountOperation() {
        ignoreAccountChanges = true;
    }

    /**
     * When an operation has ended on an account advise the receiver to listen
     * for any further account change
     */
    public static void endAccountOperation() {
        ignoreAccountChanges = false;
    }

    /**
     * Retrieve the Android account reference from the native account manager.
     * @param context the Context object
     * @return Account the NATIVE android.accounts.Account reference - the first
     * element of the account array that matches the account type for the
     * application
     */
    public static Account getNativeAccount(Context context) {

        //Needs a reference object
        if(context == null) {
            return null;
        }

        //Retrieve the AccountManager
        AccountManager am = AccountManager.get(context);
        Account[] accounts = am.getAccountsByType(context.getString(
                R.string.account_type));
        if(accounts != null && accounts.length > 0) {
            //Return the first element of the account array that matches the
            //account type for the application
            return accounts[0];
        } else {
            return null;
        }
    }
}
