/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.android.activities;

import android.accounts.Account;
import android.app.Dialog;
import android.app.Activity;
import android.os.Bundle;
import android.accounts.AccountAuthenticatorActivity;
import android.view.KeyEvent;
import android.widget.Toast;

import com.funambol.android.R;
import com.funambol.android.AppInitializer;
import com.funambol.android.activities.settings.AndroidAccountView;
import com.funambol.android.controller.AndroidController;
import com.funambol.client.controller.HomeScreenController;
import com.funambol.util.Log;

/**
 * Funambol Account Activity
 */
public class AndroidAccountScreen extends AccountAuthenticatorActivity {

    private static final String TAG = "AndroidAccountScreen";

    private AndroidDisplayManager dm = null;

    private AndroidAccountView accountView;
    private HomeScreenController homeScreenController;
    
    /**
     * Called with the activity is first created.
     */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);

        Account account = AndroidController.getNativeAccount();

        // Check if a Funambol account already exists
        if(account != null) {
            String label = getString(R.string.alert_one_account_supported_1) + " " +
                           getString(R.string.account_label) + " " +
                           getString(R.string.alert_one_account_supported_2);
            Toast.makeText(this, label, Toast.LENGTH_LONG).show();
            finish();
        }

        if(!AndroidController.isInitialized()) {
            AppInitializer initializer = AppInitializer.getInstance(this);
            initializer.initController();
        }
        
        AndroidController gc = AndroidController.getInstance();
        
        this.dm = (AndroidDisplayManager)gc.getDisplayManager();

        this.accountView = createAccountView();

        // Set the proper help text
        accountView.setHelpText(gc.getLocalization().getLanguage("message_login_screen"));

        this.homeScreenController = gc.getHomeScreenController();

        setContentView(accountView);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {

        if (keyCode == KeyEvent.KEYCODE_BACK && homeScreenController != null &&
                homeScreenController.getHomeScreen() != null) {
            // Close the main screen
            ((Activity)homeScreenController.getHomeScreen()).finish();
            finish();
            return false;
        } else {
            return super.onKeyDown(keyCode, event);
        }
    }

    @Override
    protected Dialog onCreateDialog(int id) {
        Log.trace(TAG, "onCreateDialog: " + id);
        Dialog result = null;
        if(dm != null) {
            result = dm.createDialog(id);
        }
        if(result != null) {
            return result;
        } else {
            return super.onCreateDialog(id);
        }
    }

    protected AndroidAccountView createAccountView() {
        return new AndroidAccountView(this);
    }
}
