/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.android.activities;

import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.ImageView;
import android.widget.TextView;
import android.view.ViewGroup;
import android.view.Gravity;
import android.view.animation.AnimationUtils;
import android.app.Activity;

import com.funambol.android.R;
import com.funambol.client.ui.UISyncSource;
import com.funambol.client.ui.UISyncSourceContainer;
import com.funambol.client.ui.Bitmap;
import com.funambol.client.ui.Font;
import com.funambol.client.source.AppSyncSource;
import com.funambol.util.Log;

public class AndroidButtonUISyncSource extends RelativeLayout implements UISyncSource {

    private static final String TAG = "AndroidButtonUISyncSource";

    private static final int ENABLED_TEXT_COLOR  = 0xFF000000;
    private static final int DISABLED_TEXT_COLOR = 0xFF7F7F7F;

    private String title;
    private boolean isSelected;
    private boolean isEnabled;
    private AppSyncSource appSyncSource;
    private UISyncSourceContainer container;
    private Activity activity;

    private ImageView sourceIconView;
    private TextView  titleTextView;
    private ImageView statusIconView;
    private TextView  statusTextView;

    private static final int TOP_PADDING = 8;
    private static final int BOTTOM_PADDING = 8;
    private static final int TITLE_L_PADDING = 5;
    private static final int STATUS_L_PADDING = 12;
    private static final int STATUS_ICON_R_PADDING = 4;
    private static final int SOURCE_ICON_LEFT_PADDING = 4;
    private static final int SOURCE_ICON_RIGHT_PADDING = 4;

    private static final int STATUS_FONT_SIZE = 12;
    private static final int TITLE_FONT_SIZE  = 22;

    private int enabledTitleColor = ENABLED_TEXT_COLOR;
    private int disabledTitleColor = DISABLED_TEXT_COLOR;
    private int enabledMessageColor = ENABLED_TEXT_COLOR;
    private int disabledMessageColor = DISABLED_TEXT_COLOR;

    private SetStatusIconUIThread setStatusIconUIThread = new SetStatusIconUIThread();
    private SetIconUIThread setIconUIThread = new SetIconUIThread();
    private SetStatusStringUIThread setStatusStringUIThread = new SetStatusStringUIThread();
    private SetSelectedUIThread setSelectedUIThread = new SetSelectedUIThread();
    private SetEnabledUIThread setEnabledUIThread = new SetEnabledUIThread();
    private RedrawUIThread redrawUIThread = new RedrawUIThread();

    public AndroidButtonUISyncSource(Activity activity) {
        super(activity);

        this.activity = activity;

        this.setBackgroundResource(R.drawable.sync_shape);

        sourceIconView = new ImageView(activity, null, R.style.sync_icon);
        sourceIconView.setPadding(adaptSizeToDensity(SOURCE_ICON_LEFT_PADDING),
                                  adaptSizeToDensity(TOP_PADDING),
                                  adaptSizeToDensity(SOURCE_ICON_RIGHT_PADDING),
                                  adaptSizeToDensity(BOTTOM_PADDING));

        titleTextView  = new TextView(activity, null, R.style.sync_title);
        titleTextView.setTextSize(TITLE_FONT_SIZE);
        titleTextView.setPadding(adaptSizeToDensity(TITLE_L_PADDING), 0, 0, 0);
        titleTextView.setTextColor(ENABLED_TEXT_COLOR);

        LinearLayout statusIconViewContainer = new LinearLayout(activity);
        statusIconViewContainer.setPadding(0, adaptSizeToDensity(TOP_PADDING),
                adaptSizeToDensity(STATUS_ICON_R_PADDING),
                adaptSizeToDensity(BOTTOM_PADDING));

        statusIconView = new ImageView(activity);
        statusIconView.setAdjustViewBounds(true);
        statusIconView.setMaxHeight(adaptSizeToDensity(32));
        statusIconView.setMaxWidth(adaptSizeToDensity(32));
        statusIconView.setLayoutParams(new ViewGroup.LayoutParams(LayoutParams.WRAP_CONTENT,
                                                                  LayoutParams.WRAP_CONTENT));
        statusIconViewContainer.addView(statusIconView);

        statusTextView = new TextView(activity, null, R.style.sync_message);
        statusTextView.setPadding(adaptSizeToDensity(STATUS_L_PADDING), 0, 0, 0);
        statusTextView.setTextSize(STATUS_FONT_SIZE);
        statusTextView.setTextColor(ENABLED_TEXT_COLOR);

        // Sets the linear layout
        LinearLayout ll1 = new LinearLayout(activity);
        // All items in ll1 are vertically centered
        ll1.setGravity(Gravity.CENTER_VERTICAL);
        ll1.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT,
                                                          LayoutParams.WRAP_CONTENT));

        ll1.addView(sourceIconView);
        LinearLayout ll2 = new LinearLayout(activity);
        // This is the weight parameter that we associate to the central item
        // (title and status) so it is grown to occupy all the available space
        // (pushing the iconcs on the left/right)
        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT,
                                                          LayoutParams.WRAP_CONTENT,
                                                          1.0f);
        ll2.setLayoutParams(lp);
        ll2.addView(titleTextView, lp);
        ll2.addView(statusTextView, lp);
        ll2.setOrientation(LinearLayout.VERTICAL);
        ll2.setPadding(0, adaptSizeToDensity(TOP_PADDING),
                       0, adaptSizeToDensity(BOTTOM_PADDING));
        ll1.addView(ll2);
        //ll1.addView(statusIconView);
        ll1.addView(statusIconViewContainer);

        this.addView(ll1);

        setFocusable(true);
        setFocusableInTouchMode(true);
        setClickable(true);
    }

    /**
     * Set the title
     */
    public void setTitle(String value) {
        titleTextView.setText(value);
    }

    /**
     * Set the title color when the item is enabled
     */
    public void setTitleEnabledColor(int color) {
        enabledTitleColor = color;
    }

    /**
     * Set the title color when the item is disabled
     */
    public void setTitleDisabledColor(int color) {
        disabledTitleColor = color;
    }

    /**
     * Check if the item is enabled/disabled
     */
    public boolean isDisabled() {
        return !isEnabled;
    }

    public void setIcon(Bitmap image) {
        setIconUIThread.setIcon(image);
        activity.runOnUiThread(setIconUIThread);
    }

    /**
     * Set the status icon to be used when the item is not selected
     */
    public void setStatusIcon(Bitmap image) {
        // Now set the new icon
        setStatusIconUIThread.setIcon(image);
        activity.runOnUiThread(setStatusIconUIThread);
    }

    /**
     * Set the status string
     */
    public void setStatusString(String value) {
        setStatusStringUIThread.setText(value);
        activity.runOnUiThread(setStatusStringUIThread);
    }

    public void setSelection(boolean selected, boolean fromUi) {
        setSelectedUIThread.setSelection(selected, fromUi);
        activity.runOnUiThread(setSelectedUIThread);
    }

    public void setEnabled(boolean flag) {
        setEnabledUIThread.setEnabled(flag);
        activity.runOnUiThread(setEnabledUIThread);
    }

    /**
     * Forces the current item to be re-drawn
     */
    public void redraw() {
        activity.runOnUiThread(redrawUIThread);
    }


    /**
     * Return the selected status
     */
    public boolean isSelected() {
        return isSelected;
    }

    /**
     * Returns the AppSyncSource this item represents
     */
    public AppSyncSource getSource() {
        return appSyncSource;
    }

    /**
     * Set the AppSyncSource this item represents
     */
    public void setSource(AppSyncSource source) {
        appSyncSource = source;
    }

    /**
     * Set the container this item belongs to
     */
    public void setContainer(UISyncSourceContainer container) {
        this.container = container;
    }

    /**
     * Set the item font used for the title and the status
     */
    public void setFont(Font font) {
    }

    public void syncStarted() {
        Log.trace(TAG, "syncStarted, starting a spinning icon");
    }

    public void syncEnded() {
        Log.trace(TAG, "syncEnded, stopping any spinning icon");
    }

    private class RedrawUIThread implements Runnable {
        public RedrawUIThread() {
        }

        public void run() {
            invalidate();
        }
    }

    private int adaptSizeToDensity(int size) {
        return (int)(size*getContext().getResources().getDisplayMetrics().density);
    }

    private class SetStatusStringUIThread implements Runnable {
        private String text;

        public SetStatusStringUIThread() {
        }

        public void setText(String value) {
            this.text = value;
        }

        public void run() {
            statusTextView.setText(text);
        }
    }

    private class SetIconUIThread implements Runnable {
        private Bitmap image;

        public SetIconUIThread() {
        }

        public void setIcon(Bitmap image) {
            this.image = image;
        }

        public void run() {
            if (image != null) {
                Integer id = (Integer)image.getOpaqueDescriptor();
                sourceIconView.setImageResource(id.intValue());
            } else {
                // Clear the previous image
                sourceIconView.setImageResource(0);
            }
        }
    }

    private class StartStopSpinningUIThread implements Runnable {
        private boolean stop;

        public StartStopSpinningUIThread() {
        }

        public void setStop() {
            stop = true;
        }

        public void run() {
            synchronized(statusIconView) {
                if (stop) {
                    statusIconView.clearAnimation();
                    stop = false;
                } else {
                    statusIconView.setImageResource(android.R.drawable.ic_popup_sync);
                    statusIconView.startAnimation(AnimationUtils.loadAnimation(activity, R.anim.rotate_indefinitely) );
                }
            }
        }
    }

    private class SetStatusIconUIThread implements Runnable {
        private Bitmap image;

        public SetStatusIconUIThread() {
        }

        public void setIcon(Bitmap image) {
            this.image = image;
        }

        public void run() {
            if (image != null) {
                Integer id = (Integer)image.getOpaqueDescriptor();
                statusIconView.setImageResource(id.intValue());
            } else {
                // Clear the previous image
                statusIconView.setImageResource(0);
            }
        }
    }

    private class SetSelectedUIThread implements Runnable {
        private boolean selected;
        private boolean fromUi;

        public SetSelectedUIThread() {
        }

        public void setSelection(boolean selected, boolean fromUi) {
            this.selected = selected;
            this.fromUi   = fromUi;
        }

        public void run() {
            setSelected(selected);
            if (selected) {
                setBackgroundResource(R.drawable.sync_shape_over);
                requestFocus();
            } else {
                setBackgroundResource(R.drawable.sync_shape);
            }
        }
    }

    private class SetEnabledUIThread implements Runnable {
        private boolean enabled;

        public SetEnabledUIThread() {
        }

        public void setEnabled(boolean enabled) {
            this.enabled = enabled;
        }

        public void run() {

            if (enabled) {
                titleTextView.setTextColor(enabledTitleColor);
                statusTextView.setTextColor(enabledMessageColor);
            } else {
                titleTextView.setTextColor(disabledTitleColor);
                statusTextView.setTextColor(disabledMessageColor);
            }

            setFocusable(enabled);
            setFocusableInTouchMode(enabled);
            setClickable(enabled);

            isEnabled = enabled;
        }
    }
}


