/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.android.activities.settings;

import android.app.Activity;
import android.graphics.drawable.Drawable;
import android.os.Bundle;

import com.funambol.android.R;

/**
 * Represents the Account Settings tab
 */
public class AndroidAccountSettingsTab extends AndroidSettingsTab {

    private static final String TAB_TAG = "account_settings";

    private static final int LAYOUT_ID = R.id.account_settings_tab;

    private AndroidAccountView accountView;

    public AndroidAccountSettingsTab(Activity a, Bundle state) {
        super(a, state);
        accountView = new AndroidAccountView(a);
        accountView.removeButtons();
        this.addView(accountView);
    }

    public String getTag() {
        return TAB_TAG;
    }

    public int getLayoutId() {
        return LAYOUT_ID;
    }

    public void saveSettings(SaveSettingsCallback callback) {
        accountView.saveCredential(callback);
    }

    public void cancelSettings(){
        accountView.restoreCredential();
    }

    public boolean hasChanges() {
        if(accountView.hasChanges() || !accountView.validCredentials()){
            return true;
        }else{
            return false ;
        }
    }

    public Drawable getIndicatorIcon() {
        return getResources().getDrawable(R.drawable.ic_account_tab);
    }

    public String getIndicatorLabel() {
        return localization.getLanguage("settings_account_tab");
    }
}
