/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.android.activities.settings;

import android.accounts.Account;
import android.accounts.AccountAuthenticatorActivity;
import android.accounts.AccountManager;
import android.accounts.AccountManagerFuture;
import android.accounts.OperationCanceledException;
import android.app.Activity;
import android.content.ContentValues;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import com.funambol.android.AndroidAccountManager;

import com.funambol.android.AndroidAppSyncSourceManager;
import com.funambol.android.AndroidCustomization;
import com.funambol.android.R;
import com.funambol.android.activities.AndroidDisplayManager;
import com.funambol.android.controller.AndroidController;
import com.funambol.android.controller.AndroidAccountScreenController;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.localization.Localization;

import com.funambol.client.controller.Controller;
import com.funambol.client.controller.AccountScreenController;
import com.funambol.client.controller.UISyncSourceController;
import com.funambol.client.customization.Customization;
import com.funambol.client.source.AppSyncSource;
import com.funambol.client.source.AppSyncSourceManager;
import com.funambol.client.ui.AccountScreen;
import com.funambol.client.ui.Bitmap;
import com.funambol.client.ui.DisplayManager;
import com.funambol.client.ui.Font;
import com.funambol.client.ui.UISyncSource;
import com.funambol.client.ui.UISyncSourceContainer;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncListener;
import com.funambol.util.Log;
import java.io.IOException;
import java.util.Enumeration;

/**
 * Represents a generic settings tab
 */
public class AndroidAccountView extends LinearLayout implements AccountScreen,
        UISyncSource, UISyncSourceContainer{

    private static final String TAG_LOG = "AndroidAccountView";
    
    public static final int CHECKING_CREDENTIALS_DIALOG_ID = 1000;

    private TextView helpText;
    private EditText usrText;
    private EditText pswText;
    private EditText urlText;
    private Button   loginButton;

    private boolean validCredential = true;

    private LinearLayout buttonsBar;

    private Activity activity;

    private SpinnerTriggerUIThread spinnerTriggerUIThread;
    private static int pDialogId = -1;

    private AppSyncSource configSource;
    private AccountScreenController controller;
    private Customization customization;

    private EnableSaveUIThread enableSaveUIThread = new EnableSaveUIThread();
    
    // Used to notify the settings screen when the account settings are saved correctly
    SaveSettingsCallback saveSettingsCallback = null;

    private AndroidDisplayManager displayManager;

    private Localization localization;

    public AndroidAccountView(Activity a) {

        super(a);
        this.activity = a;
        spinnerTriggerUIThread = new SpinnerTriggerUIThread(activity);
        
        customization = AndroidCustomization.getInstance();

        View.inflate(a, R.layout.account, this);

        helpText = (TextView) findViewById(R.id.account_help_text);
        usrText = (EditText) findViewById(R.id.username);
        pswText = (EditText) findViewById(R.id.password);
        urlText = (EditText) findViewById(R.id.syncUrl);
        
        int width = usrText.getWidth();

        usrText.setMaxWidth(width);
        pswText.setMaxWidth(width);
        urlText.setMaxWidth(width);

        loginButton = (Button)findViewById(R.id.login_button);
        loginButton.setOnClickListener(new OkListener());
        loginButton.setId(1);
        
        buttonsBar = (LinearLayout)findViewById(R.id.buttons_bar);

        // We must initialize the controller here
        AndroidController gc = AndroidController.getInstance();
        controller = new AndroidAccountScreenController(gc, this);
        gc.setAccountScreenController(controller);
        
        controller.initScreen();

        this.localization = gc.getLocalization();
        this.displayManager = (AndroidDisplayManager)gc.getDisplayManager();

        initialize(controller);
    }

    public void setActivity(Activity a) {
        this.activity = a;
    }

    public void initialize(AccountScreenController controller) {
        this.controller = controller;

        // Bind the source to the controller and view objects
        Controller gc = controller.getController();
        AppSyncSourceManager appSyncSourceManager = gc.getAppSyncSourceManager();
        configSource = appSyncSourceManager.getSource(AndroidAppSyncSourceManager.CONFIG_ID);
       
        // If a config source is not available, then the client must be
        // configured without credentials checking.
        if (configSource != null) {
            Log.debug(TAG_LOG, "Registering source controller");
            UISyncSourceController uiSourceController = configSource.getUISyncSourceController();
            if(uiSourceController == null) {
                uiSourceController = new UISyncSourceController(gc.getCustomization(),
                                                                gc.getLocalization(),
                                                                appSyncSourceManager,
                                                                gc, configSource);
            }
            configSource.setUISyncSourceController(uiSourceController);
            uiSourceController.setUISyncSource(this);
            uiSourceController.disableStatusAnimation();
            configSource.getSyncSource().setListener(uiSourceController);
        }

        // Remove the server url field if needed
        if (!customization.syncUriEditable()) {
            ViewGroup table = (ViewGroup)findViewById(R.id.account_table);
            View serverUrlRow = findViewById(R.id.account_sync_url_row);
            table.removeView(serverUrlRow);
        }
    }

    public void setIcon(Bitmap arg0) {
    }

    public void restoreCredential() {
        if (!validCredential){
            controller.resetValues();
        }
    }

    public boolean validCredentials() {
        return validCredential;
    }

    /**
     * A call-back for when the user presses the ok button.
     */
    private class OkListener implements OnClickListener {
        public void onClick(View v) {
             saveAndCheck();
        }
    }

    public void saveCredential(SaveSettingsCallback callback) {
        
        saveSettingsCallback = callback;
        if(hasChanges() || !validCredential) {
            ((Activity)getScreen()).runOnUiThread(new AlertUIThread(
                    localization.getLanguage("alert_contacts_reset"),
                    new Runnable() {
                        public void run() {
                            saveAndCheck();
                        }
                    },
                    new Runnable() {
                        public void run() {

                        }
                    }));
        } else {
            saveSettingsCallback.saveSettingsResult(true);
        }
    }
    
    public boolean hasChanges() {
        String serverUri;
        if (customization.syncUriEditable()) {
            serverUri = urlText.getText().toString();
        } else {
            serverUri = customization.getServerUriDefault();
        }
        return controller.hasChanges(serverUri, usrText.getText().toString(),
                pswText.getText().toString());
    }

    private void saveAndCheck() {
        String serverUri;
        if (customization.syncUriEditable()) {
            serverUri = urlText.getText().toString();
        } else {
            serverUri = customization.getServerUriDefault();
        }
        controller.saveAndCheck(serverUri, usrText.getText().toString(),
                pswText.getText().toString());
    }

    public void disableSave() {
        enableSave(false);
    }

    public void enableSave() {
        enableSave(true);
    }

    public void enableCancel() {  
    }

    public void disableCancel() { 
    }

    public void checkFailed() {
        validCredential = false;
    }

    private void enableSave(boolean value) {
        enableSaveUIThread.setValue(value);
        activity.runOnUiThread(enableSaveUIThread);
    }

    public void removeButtons(){
       ((LinearLayout)this.getChildAt(0)).removeView(buttonsBar);
    }

    public void checkSucceeded() {

        AccountManager am = AccountManager.get(activity);
        Account existingAccount = AndroidController.getNativeAccount();

        if (existingAccount != null) {
            AndroidAccountManager.beginAccountOperation();
            try {
                AccountManagerFuture<Boolean> result = am.removeAccount(
                        existingAccount, null, null);
                result.getResult();

                // Reset source anchors
                Log.debug(TAG_LOG, "Reset source anchors");
                Controller gc = controller.getController();
                Enumeration sources = gc.getAppSyncSourceManager()
                        .getWorkingSources();

                while(sources.hasMoreElements()) {
                    AppSyncSource ass = (AppSyncSource)sources.nextElement();
                    ass.getConfig().setLastSyncStatus(SyncListener.SUCCESS);
                    ass.getUISyncSource().setStatusIcon(null);
                    
                    Log.debug(TAG_LOG, "Reset anchors for source: " + ass.getName());
                    SourceConfig sc = ass.getSyncSource().getConfig();
                    sc.setLastAnchor(0);
                    sc.setNextAnchor(0);

                    ass.getConfig().saveSourceSyncConfig();
                }
            } catch(Exception ex) {
                Log.error(TAG_LOG, "Failed to remove existing account: " + ex); 
            }
            AndroidAccountManager.endAccountOperation();
        }
        
        // Add the account if necessary
        // If the account is not available, we need to create it
       
        Log.info(TAG_LOG, "Adding account explicitly");

        // The account label must be the username
        String accountLabel = usrText.getText().toString();

        //Set the account type
        String accountType = activity.getString(R.string.account_type);

        // Create the new account
        Account account = new Account(accountLabel, accountType);

        if (am.addAccountExplicitly(account, null, null)) {

            Bundle result = new Bundle();
            result.putString(AccountManager.KEY_ACCOUNT_NAME, accountLabel);
            result.putString(AccountManager.KEY_ACCOUNT_TYPE, accountType);
            ((AccountAuthenticatorActivity)this.activity).setAccountAuthenticatorResult(result);
        
            // Set the new created account contacts to be displayed in the addressbook
            ContentValues cv = new ContentValues();
            cv.put(ContactsContract.Settings.ACCOUNT_NAME, accountLabel);
            cv.put(ContactsContract.Settings.ACCOUNT_TYPE, accountType);
            cv.put(ContactsContract.Settings.UNGROUPED_VISIBLE, 1);
            activity.getContentResolver().insert(ContactsContract.Settings.CONTENT_URI, cv);
        }

        validCredential = true;
        // Notify the settings screen
        if(saveSettingsCallback != null) {
            saveSettingsCallback.saveSettingsResult(true);
        } else {
            activity.finish();
        }
    }

    public void setSyncUrl(String originalUrl) {
        if (customization.syncUriEditable()) {
            urlText.setText(originalUrl);
        }
    }

    public void setUsername(String originalUser) {
        usrText.setText(originalUser);
    }

    public void setPassword(String originalPassword) {
        pswText.setText(originalPassword);
    }

    public void setHelpText(String text) {
        helpText.setText(text);
    }

    public Object getScreen() {
        return activity;
    }

      ///////////////// UISyncSource implementation ////////////////////
    // All the methods are empty because we don't show any message at the moment
    // but the ones to start/stop the spinning wheel
    public void setTitle(String arg0) {
    }

    public void setTitleEnabledColor(int arg0) {
    }

    public void setTitleDisabledColor(int arg0) {
    }

    public boolean isDisabled() {
        return false;
    }

    public void setStatusIcon(Bitmap arg0) {
    }

    public void setStatusString(String value) {
    }

    public void setSelection(boolean selected, boolean fromUi) {
    }

    public void setEnabled(boolean flag) {
    }

    public void setSelected(boolean selected) {
    }

    public boolean isSelected() {
        return false;
    }

    public AppSyncSource getSource() {
        return configSource;
    }

    public void setSource(AppSyncSource arg0) {
    }

    public void redraw() {
    }

    public void setContainer(UISyncSourceContainer arg0) {
    }

    public void setFont(Font arg0) {
    }

    public void syncStarted() {
        Log.debug(TAG_LOG, "Sync Started");
        spinnerTriggerUIThread.setValue(true);
        activity.runOnUiThread(spinnerTriggerUIThread);

    }

    public void syncEnded() {
        Log.debug(TAG_LOG, "Sync Ended");
        spinnerTriggerUIThread.setValue(false);
        activity.runOnUiThread(spinnerTriggerUIThread);
    }

    private class SpinnerTriggerUIThread implements Runnable {

        private boolean value;

        private DisplayManager dm = null;

        private Localization localization;

        public SpinnerTriggerUIThread(Activity activity) {

            AndroidController gc = AndroidController.getInstance();

            this.dm = gc.getDisplayManager();
            this.localization = gc.getLocalization();
        }

        public void setValue(boolean value) {
            this.value = value;
        }

        public void run() {
            if (value) {
                String text = localization.getLanguage("checking_credentials_title");
                pDialogId = dm.showProgressDialog(AndroidAccountView.this, text);
            } else if(pDialogId != -1){
                dm.dismissProgressDialog(AndroidAccountView.this, pDialogId);
                pDialogId = -1;
            }
        }
    }

    private class EnableSaveUIThread implements Runnable {
        private boolean value;

        public EnableSaveUIThread() {
        }

        public void setValue(boolean value) {
            this.value = value;
        }

        public void run() {
            loginButton.setEnabled(value);
        }
    }

    private class AlertUIThread implements Runnable {
        
        private String question;
        private Runnable yesAction;
        private Runnable noAction;

        public AlertUIThread(String question,
                             Runnable yesAction,
                             Runnable noAction) {
            this.question = question;
            this.yesAction = yesAction;
            this.noAction = noAction;
        }

        public void run() {
            // Show alert
            displayManager.askYesNoQuestion(AndroidAccountView.this, question,
                    yesAction, noAction, 0);
        }
    }

}
