/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.android.source.media.picture;

import java.util.Enumeration;

import android.content.Context;
import android.net.Uri;
import android.database.Cursor;
import android.provider.MediaStore.MediaColumns;

import com.funambol.android.source.media.MediaTracker;
import com.funambol.android.providers.PicturesContentProvider;
import com.funambol.storage.StringKeyValueStore;

import com.funambol.client.configuration.Configuration;
import com.funambol.client.source.AppSyncSource;
import com.funambol.syncml.client.TrackerException;
import com.funambol.syncml.spds.SyncException;

import com.funambol.util.Log;


public class PictureTracker extends MediaTracker {
    private static final String TAG = "PictureTracker";
    private AppSyncSource appSource;
    private Configuration configuration;

    public PictureTracker(Context context, StringKeyValueStore status, AppSyncSource appSource,
                          Configuration configuration) {
        super(context, status);
        this.appSource = appSource;
        this.configuration = configuration;
    }

    @Override
    public void begin(int syncMode) throws TrackerException {
        super.begin(syncMode);
        PictureAppSyncSourceConfig config = (PictureAppSyncSourceConfig)appSource.getConfig();
        // Filter outgoing items if the user enabled the filter
        if (! config.getIncludeOlderMedia()) {
            // Remove the old items from the list of changes
            Enumeration adds = newItems.keys();
            while(adds.hasMoreElements()) {
                String key = (String)adds.nextElement();
                Uri itemUri = Uri.withAppendedPath(getProviderUri(), key);
                // Get the item meta information
                Cursor cursor = context.getContentResolver().query(itemUri, null, null, null, null);
                if (cursor == null) {
                    Log.error(TAG, "Item not found " + itemUri);
                    throw new SyncException(SyncException.CLIENT_ERROR, "Cannot get meta info for item: " + itemUri);
                }

                int added = 0;
                try {
                    int addedIndex = cursor.getColumnIndexOrThrow(MediaColumns.DATE_ADDED);
                    cursor.moveToFirst();
                    if (cursor.getCount() != 1) {
                        Log.error(TAG, "Item not found, cannot check date for " + itemUri);
                        throw new TrackerException("Cannot get meta info for item: " + itemUri);
                    }
                    added = cursor.getInt(addedIndex);

                    if (added < (configuration.getFirstRunTimestamp()/1000)) {
                        Log.info(TAG, "Filtering out old picture: " + key);
                        newItems.remove(key);
                    }
                } catch (Exception e) {
                    Log.error(TAG, "Cannot check if item must be filtered out", e);
                } finally {
                    cursor.close();
                }
            }
        }
    }

    protected Uri getProviderUri() {
        return PicturesContentProvider.EXTERNAL_CONTENT_URI;
    }
}


