/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.android.source.pim.calendar;

import java.util.ArrayList;

import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.ArrayAdapter;
import android.app.Activity;

import com.funambol.android.activities.settings.AndroidSettingsUISyncSource;
import com.funambol.util.Log;


public class CalendarSettingsUISyncSource extends AndroidSettingsUISyncSource {

    private static final String TAG = "CalendarSettingsUISyncSource";

    private String  calendarNameInitialValue;
    private Spinner calendars;
    private ArrayAdapter calendarsChoices;
    private Activity activity;
    private CalendarManager cm;
    private ArrayList<CalendarManager.CalendarDescriptor> cals;

    public CalendarSettingsUISyncSource(Activity activity) {

        super(activity);
        this.activity = activity;

    }

    @Override
    public void saveCustomSettings() {
        Log.trace(TAG, "Saving custom settings for calendar source");
        CalendarAppSyncSourceConfig config = (CalendarAppSyncSourceConfig)appSyncSource.getConfig();
        int pos = calendars.getSelectedItemPosition();
        if (cals != null) {
            CalendarManager.CalendarDescriptor calDes = cals.get(pos);
            long calId = calDes.getId();
            config.setCalendarId(calId);
        }
    }

    @Override
    public void loadCustomSettings() {
        Log.trace(TAG, "Loading custom settings for calendar source");
        CalendarAppSyncSourceConfig config = (CalendarAppSyncSourceConfig)appSyncSource.getConfig();
        long calId = config.getCalendarId();
        // Scan available names to see if this one is still available
    }

    @Override
    public boolean hasChanges() {
        boolean changes = super.hasChanges();
        if (calendarNameInitialValue == null) {
            changes = true;
        } else {
            changes |= calendarNameInitialValue.equals(calendars.getSelectedItem());
        }
        return changes;
    }

    @Override
    public void layout() {

        super.layout();

        calendars = new Spinner(activity); 
        cm = new CalendarManager(activity, appSyncSource);
        updateListOfAvailableCalendars();

        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT,
                    LayoutParams.WRAP_CONTENT);
        mainLayout.addView(calendars, lp);
    }

    private void updateListOfAvailableCalendars() {
        calendarsChoices = new ArrayAdapter<CharSequence>(activity, android.R.layout.simple_spinner_item);

        ArrayList<CalendarManager.CalendarDescriptor> cals = cm.getCalendars();
        if (cals != null) {
            for(CalendarManager.CalendarDescriptor cd : cals) {
                calendarsChoices.add(cd.getName());
            }
        } else {
            Log.info(TAG, "No calendars to sync");
        }

        calendarsChoices.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        calendars.setAdapter(calendarsChoices);
    }
}


